/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#include "manager.h"

Manager::Manager(Cache* cache, QObject* parent, const char* name)
	: QObject(parent, name)
{
	connect(cache, SIGNAL(rssAvailable(QString, ItemList)),
	        this, SLOT(receiveItems(QString, ItemList)));
	connect(cache, SIGNAL(progress(QString, int)),
	        this, SLOT(progress(QString, int)));
	connect(cache, SIGNAL(urlModified(QString, QString)),
	        this, SLOT(modifyUrl(QString, QString)));
	
	this->cache = cache;
	this->url = "";
	this->index = 0;
}

void
Manager::modifyUrl(QString prev, QString curr)
{
	if (url == prev)
		url = curr;
}

void
Manager::setSites(SiteList sites)
{
	int urlindex = 0;
	if (this->urls.contains(this->url))
		urlindex = this->urls.findIndex(this->url);
	
	this->urls.clear();
	for (unsigned int i = 0; i < sites.count(); i++) {
		Site site = sites[i];
		if (site.enabled)
			this->urls.append(site.url);
	}
	
	if (!this->urls.contains(this->url)) {
		if (urlindex >= (int)this->urls.count()) {
			urlindex = 0;
		}
		this->url = this->urls[urlindex];
		this->index = 0;
	}
}

void
Manager::jump(QString url)
{
	if (this->urls.contains(url))
		this->url = url;
}

void
Manager::next()
{
	if (this->urls.count() == 0) {
		emit noItems();
		return;
	}
	
	if (this->url == "") {
		this->url = this->urls.first();
		this->index = 0;
	} else {
		this->index++;
	}
	this->getData();
}

void
Manager::previous()
{
	if (this->urls.count() == 0) {
		emit noItems();
		return;
	}
	
	if (this->url == "") {
		this->url = this->urls.last();
		this->index = -1;
	} else {
		this->index--;
		if (this->index == -1) {
			int urlindex = this->urls.findIndex(this->url);
			urlindex--;
			if (urlindex == -1)
				urlindex = this->urls.count() - 1;
			this->url = this->urls[urlindex];
		}
	}
	this->getData();
}

void
Manager::getData()
{
	int index = this->urls.findIndex(this->url);
	int prev = index - 1;
	if (prev == -1)
		prev = this->urls.count() - 1;
	int next = index + 1;
	if ((unsigned int)next == this->urls.count())
		next = 0;
	this->cache->retrieve(this->urls[index]);
	this->cache->retrieve(this->urls[next]);
	this->cache->retrieve(this->urls[prev]);
}

void
Manager::receiveItems(QString url, ItemList items)
{
	if (url != this->url)
		return;
	
	if (index < 0)
		this->index = items.count() - 1;
	
	if (index < 0) {
		index = 0;
		previous();
		return;
	}
	
	if (this->index >= (int)items.count()) {
		int urlindex = this->urls.findIndex(this->url);
		urlindex++;
		if (urlindex == (int)this->urls.count())
			urlindex = 0;
		this->url = this->urls[urlindex];
		this->index = 0;
		this->cache->retrieve(this->url);
		return;
	}
	
	emit itemAvailable(items[index]);
}

void
Manager::progress(QString url, int)
{
	if (url == this->url)
		emit itemLoading();
}
